@@ -8,9 +8,10 @@ Todo:
 from typing import Callable, Union
 
 import numpy as np
+from genbase import Readable
 from instancelib.instances.memory import MemoryBucketProvider
 
-from ..default import Readable
+from ..utils import default_tokenizer
 
 
 def as_n_dimensional(vectors: Union[np.ndarray, list, MemoryBucketProvider],
@@ -60,7 +61,7 @@ def as_2d(vectors: Union[np.ndarray, list, MemoryBucketProvider], method: str =
 
 def as_3d(vectors: Union[np.ndarray, list, MemoryBucketProvider], method: str = 'pca', **kwargs) -> np.ndarray:
     """Summarize vectors in 3 dimensions."""
-    return as_n_dimensional(vectors=vectors, n=2, method=method, **kwargs)
+    return as_n_dimensional(vectors=vectors, n=3, method=method, **kwargs)
 
 
 class Embedder(Readable):
@@ -122,8 +123,7 @@ class SentenceTransformer(Embedder):
         .. _Sentence Transformers:
             https://github.com/UKPLab/sentence-transformers
         """
-        from sentence_transformers import \
-            SentenceTransformer as SentTransformer
+        from sentence_transformers import SentenceTransformer as SentTransformer
         self.model = SentTransformer(model_name)
         super().__init__(lambda x: self.model.encode(x, **kwargs))
 
@@ -139,6 +139,8 @@ class CountVectorizer(Embedder):
             https://scikit-learn.org/stable/modules/generated/sklearn.feature_extraction.text.CountVectorizer.html
         """
         from sklearn.feature_extraction.text import CountVectorizer as Count
+        if "tokenizer" not in kwargs:
+            kwargs["tokenizer"] = default_tokenizer
         self.model = Count(**kwargs)
         super().__init__(lambda x: self.model.fit_transform(x).toarray())
 
@@ -154,5 +156,7 @@ class TfidfVectorizer(Embedder):
             https://scikit-learn.org/stable/modules/generated/sklearn.feature_extraction.text.TfidfVectorizer.html
         """
         from sklearn.feature_extraction.text import TfidfVectorizer as Tfidf
+        if "tokenizer" not in kwargs:
+            kwargs["tokenizer"] = default_tokenizer
         self.model = Tfidf(**kwargs)
         super().__init__(lambda x: self.model.fit_transform(x).toarray())
